=== modified file 'NEWS'
--- NEWS	2007-07-15 11:24:18 +0000
+++ NEWS	2007-07-20 16:57:41 +0000
@@ -2,13 +2,13 @@
 
   BUGFIXES:
 
-    * ``bzr init`` should only connect to the remote location one time.  We
+    * ``bzr init`` should connect to the remote location one time only.  We
       have been connecting several times because we forget to pass around the
       Transport object. This modifies ``BzrDir.create_branch_convenience``,
-      so that we can pass in the Transport that we already have.
+      so that we can give it the Transport we already have.
       (John Arbash Meinel, Vincent Ladeuil, #111702)
 
-    * Get rid of sftp connection cache.
+    * Get rid of sftp connection cache (get rid of the FTP one too).
       (Vincent Ladeuil, #43731)
 
     * bzr branch {local|remote} remote don't try to create a working tree
@@ -31,9 +31,6 @@
     * ``info`` now formats locations more nicely and lists "submit" and
       "public" branches (Aaron Bentley)
 
-    * Get rid of FTP connection cache.
-     (Vincent Ladeuil)
-
     * New ``pack`` command that will trigger database compression within
       the repository (Robert Collins)
 
@@ -65,7 +62,7 @@
 
     * Refactoring of transport classes connected to a remote server.
       ConnectedTransport is a new class that serves as a basis for all
-      transports needing to connet to a remote server.  transport.split_url
+      transports needing to connect to a remote server.  transport.split_url
       have been deprecated, use the static method on the object instead. URL
       tests have been refactored too.
       (Vincent Ladeuil)

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2007-07-15 11:24:18 +0000
+++ bzrlib/builtins.py	2007-07-20 16:57:41 +0000
@@ -595,27 +595,8 @@
             tree_to = None
             branch_to = Branch.open_containing(directory)[0]
 
-        reader = None
-        # The user may provide a bundle or branch as 'location' We first try to
-        # identify a bundle, if it's not, we try to preserve connection used by
-        # the transport to access the branch.
         if location is not None:
-            url = urlutils.normalize_url(location)
-            url, filename = urlutils.split(url, exclude_trailing_slash=False)
-            location_transport = transport.get_transport(url)
-            if filename:
-                try:
-                    read_bundle = bundle.read_mergeable_from_transport
-                    # There may be redirections but we ignore the intermediate
-                    # and final transports used
-                    mergeable, t = read_bundle(location_transport, filename)
-                except errors.NotABundle:
-                    # Continue on considering this url a Branch but adjust the
-                    # location_transport
-                    location_transport = location_transport.clone(filename)
-            else:
-                # A directory was provided, location_transport is correct
-                pass
+            mergeable, location_transport = _get_bundle_helper(location)
 
         stored_loc = branch_to.get_parent()
         if location is None:
@@ -2672,32 +2653,18 @@
         other_transport = None
         other_revision_id = None
         possible_transports = []
-        # The user may provide a bundle or branch as 'branch' We first try to
-        # identify a bundle, if it's not, we try to preserve connection used by
-        # the transport to access the branch.
+
         if branch is not None:
-            url = urlutils.normalize_url(branch)
-            url, filename = urlutils.split(url, exclude_trailing_slash=False)
-            other_transport = transport.get_transport(url)
-            if filename:
-                try:
-                    read_bundle = bundle.read_mergeable_from_transport
-                    # There may be redirections but we ignore the intermediate
-                    # and final transports used
-                    mergeable, t = read_bundle(other_transport, filename)
-                except errors.NotABundle:
-                    # Continue on considering this url a Branch but adjust the
-                    # other_transport
-                    other_transport = other_transport.clone(filename)
-                else:
-                    if revision is not None:
-                        raise errors.BzrCommandError('Cannot use -r with merge'
-                                                     ' directives or bundles')
-                    other_revision_id = mergeable.install_revisions(
-                        tree.branch.repository)
-                    revision = [RevisionSpec.from_string(
-                        'revid:' + other_revision_id)]
-                possible_transports.append(other_transport)
+            mergeable, other_transport = _get_bundle_helper(branch)
+            if mergeable:
+                if revision is not None:
+                    raise errors.BzrCommandError('Cannot use -r with merge'
+                                                 ' directives or bundles')
+                other_revision_id = mergeable.install_revisions(
+                    tree.branch.repository)
+                revision = [RevisionSpec.from_string('revid:'
+                                                     + other_revision_id)]
+            possible_transports.append(other_transport)
 
         if revision is None \
                 or len(revision) < 1 or revision[0].needs_branch():
@@ -2764,7 +2731,7 @@
         try:
             try:
                 conflict_count = _merge_helper(
-                    other, base, possible_transports,
+                    other, base,
                     other_rev_id=other_revision_id,
                     check_clean=(not force),
                     merge_type=merge_type,
@@ -2773,7 +2740,8 @@
                     pull=pull,
                     this_dir=directory,
                     pb=pb, file_list=interesting_files,
-                    change_reporter=change_reporter)
+                    change_reporter=change_reporter,
+                    possible_transports=possible_transports)
             finally:
                 pb.finished()
             if conflict_count != 0:
@@ -3052,7 +3020,7 @@
                                              " or specified.")
             display_url = urlutils.unescape_for_display(parent,
                                                         self.outf.encoding)
-            self.outf.write("Using last location: " + display_url + '\n')
+            self.outf.write("Using last location: " + display_url + "\n")
 
         remote_branch = Branch.open(other_branch)
         if remote_branch.base == local_branch.base:
@@ -3753,7 +3721,7 @@
 
 
 # command-line interpretation helper for merge-related commands
-def _merge_helper(other_revision, base_revision, possible_transports=None,
+def _merge_helper(other_revision, base_revision,
                   check_clean=True, ignore_zero=False,
                   this_dir=None, backup_files=False,
                   merge_type=None,
@@ -3761,7 +3729,8 @@
                   pull=False,
                   pb=DummyProgress(),
                   change_reporter=None,
-                  other_rev_id=None):
+                  other_rev_id=None,
+                  possible_transports=None):
     """Merge changes into a tree.
 
     base_revision
@@ -3871,6 +3840,34 @@
         cur_transport.ensure_base()
 
 
+def _get_bundle_helper(location):
+    """Get a bundle if 'location' points to one.
+
+    Try try to identify a bundle and returns its mergeable form. If it's not,
+    we return the tried transport anyway so that it can reused to access the
+    branch
+
+    :param location: can point to a bundle or a branch.
+
+    :return: mergeable, transport
+    """
+    mergeable = None
+    url = urlutils.normalize_url(location)
+    url, filename = urlutils.split(url, exclude_trailing_slash=False)
+    location_transport = transport.get_transport(url)
+    if filename:
+        try:
+            # There may be redirections but we ignore the intermediate
+            # and final transports used
+            read = bundle.read_mergeable_from_transport
+            mergeable, t = read(location_transport, filename)
+        except errors.NotABundle:
+            # Continue on considering this url a Branch but adjust the
+            # location_transport
+            location_transport = location_transport.clone(filename)
+    return mergeable, location_transport
+
+
 # Compatibility
 merge = _merge_helper
 

=== modified file 'bzrlib/tests/blackbox/test_branch.py'
--- bzrlib/tests/blackbox/test_branch.py	2007-06-27 19:13:50 +0000
+++ bzrlib/tests/blackbox/test_branch.py	2007-07-20 16:57:41 +0000
@@ -22,6 +22,7 @@
 from bzrlib import branch, bzrdir
 from bzrlib.repofmt.knitrepo import RepositoryFormatKnit1
 from bzrlib.tests.blackbox import ExternalBase
+from bzrlib.tests.test_sftp_transport import TestCaseWithSFTPServer
 from bzrlib.workingtree import WorkingTree
 
 
@@ -89,3 +90,26 @@
         self.assertTrue(pushed_repo.has_revision('b-1'))
 
 
+class TestRemoteBranch(TestCaseWithSFTPServer):
+
+    def setUp(self):
+        super(TestRemoteBranch, self).setUp()
+        tree = self.make_branch_and_tree('branch')
+        self.build_tree_contents([('branch/file', 'file content\n')])
+        tree.add('file')
+        tree.commit('file created')
+
+    def test_branch_local_remote(self):
+        self.run_bzr(['branch', 'branch', self.get_url('remote')])
+        t = self.get_transport()
+        # Ensures that no working tree what created remotely
+        self.assertFalse(t.has('remote/file'))
+
+    def test_branch_remote_remote(self):
+        # Light cheat: we access the branch remotely
+        self.run_bzr(['branch', self.get_url('branch'),
+                      self.get_url('remote')])
+        t = self.get_transport()
+        # Ensures that no working tree what created remotely
+        self.assertFalse(t.has('remote/file'))
+

=== modified file 'bzrlib/tests/commands/test_branch.py'
--- bzrlib/tests/commands/test_branch.py	2007-07-15 10:44:12 +0000
+++ bzrlib/tests/commands/test_branch.py	2007-07-20 16:57:41 +0000
@@ -16,7 +16,7 @@
 
 
 from bzrlib.builtins import cmd_branch
-from bzrlib.tests.TransportUtil import TestCaseWithConnectionHookedTransport
+from bzrlib.tests.transport_util import TestCaseWithConnectionHookedTransport
 
 
 class TestBranch(TestCaseWithConnectionHookedTransport):
@@ -25,20 +25,19 @@
         super(TestBranch, self).setUp()
         self.make_branch_and_tree('branch')
         self.install_hooks()
-        self.addCleanup(self.reset_hooks)
 
     def test_branch_remote_local(self):
         cmd = cmd_branch()
-        cmd.run(self.get_url() + 'branch', 'local')
+        cmd.run(self.get_url('branch'), 'local')
         self.assertEquals(1, len(self.connections))
 
     def test_branch_local_remote(self):
         cmd = cmd_branch()
-        cmd.run('branch', self.get_url() + 'remote')
+        cmd.run('branch', self.get_url('remote'))
         self.assertEquals(1, len(self.connections))
 
     def test_branch_remote_remote(self):
         cmd = cmd_branch()
-        cmd.run(self.get_url() + '/branch', self.get_url() + 'remote')
+        cmd.run(self.get_url('branch'), self.get_url('remote'))
         self.assertEquals(2, len(self.connections))
 

=== modified file 'bzrlib/tests/commands/test_cat.py'
--- bzrlib/tests/commands/test_cat.py	2007-07-15 11:17:19 +0000
+++ bzrlib/tests/commands/test_cat.py	2007-07-20 16:57:41 +0000
@@ -18,7 +18,8 @@
 
 from bzrlib.builtins import cmd_cat
 from bzrlib.tests import StringIOWrapper
-from bzrlib.tests.TransportUtil import TestCaseWithConnectionHookedTransport
+from bzrlib.tests.transport_util import TestCaseWithConnectionHookedTransport
+
 
 class TestCat(TestCaseWithConnectionHookedTransport):
 
@@ -35,16 +36,20 @@
         self.addCleanup(restore_stdout)
 
     def test_cat(self):
+        # FIXME: sftp raises ReadError instead of NoSuchFile when probing for
+        # branch/foo/.bzr/branch-format when used with the paramiko test
+        # server.
+        from bzrlib.tests import TestSkipped
+        raise TestSkipped
         wt1 = self.make_branch_and_tree('branch')
-        file('branch/foo', 'wb').write('foo')
+        self.build_tree_contents([('branch/foo', 'foo')])
         wt1.add('foo')
         wt1.commit('add foo')
 
         self.install_hooks()
-        self.addCleanup(self.reset_hooks)
 
         cmd = cmd_cat()
-        cmd.run(self.get_url() + 'branch/foo')
+        cmd.run(self.get_url('branch/foo'))
         self.assertEquals(1, len(self.connections))
         self.assertEquals('foo', self.outf.getvalue())
 

=== modified file 'bzrlib/tests/commands/test_checkout.py'
--- bzrlib/tests/commands/test_checkout.py	2007-07-15 11:17:19 +0000
+++ bzrlib/tests/commands/test_checkout.py	2007-07-20 16:57:41 +0000
@@ -16,7 +16,8 @@
 
 
 from bzrlib.builtins import cmd_checkout
-from bzrlib.tests.TransportUtil import TestCaseWithConnectionHookedTransport
+from bzrlib.tests.transport_util import TestCaseWithConnectionHookedTransport
+
 
 class TestCheckout(TestCaseWithConnectionHookedTransport):
 
@@ -24,9 +25,8 @@
         self.make_branch_and_tree('branch1')
 
         self.install_hooks()
-        self.addCleanup(self.reset_hooks)
 
         cmd = cmd_checkout()
-        cmd.run(self.get_url() + 'branch1', 'local')
+        cmd.run(self.get_url('branch1'), 'local')
         self.assertEquals(1, len(self.connections))
 

=== modified file 'bzrlib/tests/commands/test_init.py'
--- bzrlib/tests/commands/test_init.py	2007-06-06 08:42:07 +0000
+++ bzrlib/tests/commands/test_init.py	2007-07-20 16:57:41 +0000
@@ -16,14 +16,14 @@
 
 
 from bzrlib.builtins import cmd_init
-from bzrlib.tests.TransportUtil import TestCaseWithConnectionHookedTransport
+from bzrlib.tests.transport_util import TestCaseWithConnectionHookedTransport
+
 
 class TestInit(TestCaseWithConnectionHookedTransport):
 
     def setUp(self):
         super(TestInit, self).setUp()
         self.install_hooks()
-        self.addCleanup(self.reset_hooks)
 
     def test_init(self):
         cmd = cmd_init()

=== modified file 'bzrlib/tests/commands/test_init_repository.py'
--- bzrlib/tests/commands/test_init_repository.py	2007-06-06 08:42:07 +0000
+++ bzrlib/tests/commands/test_init_repository.py	2007-07-20 16:57:41 +0000
@@ -16,14 +16,14 @@
 
 
 from bzrlib.builtins import cmd_init_repository
-from bzrlib.tests.TransportUtil import TestCaseWithConnectionHookedTransport
+from bzrlib.tests.transport_util import TestCaseWithConnectionHookedTransport
+
 
 class TestInitRepository(TestCaseWithConnectionHookedTransport):
 
     def setUp(self):
         super(TestInitRepository, self).setUp()
         self.install_hooks()
-        self.addCleanup(self.reset_hooks)
 
     def test_init_repository(self):
         cmd = cmd_init_repository()

=== modified file 'bzrlib/tests/commands/test_merge.py'
--- bzrlib/tests/commands/test_merge.py	2007-07-15 11:17:19 +0000
+++ bzrlib/tests/commands/test_merge.py	2007-07-20 16:57:41 +0000
@@ -17,7 +17,8 @@
 
 from bzrlib.builtins import cmd_merge
 from bzrlib.tests import StringIOWrapper
-from bzrlib.tests.TransportUtil import TestCaseWithConnectionHookedTransport
+from bzrlib.tests.transport_util import TestCaseWithConnectionHookedTransport
+
 
 class TestMerge(TestCaseWithConnectionHookedTransport):
 
@@ -29,11 +30,10 @@
         wt2.commit('empty commit too')
 
         self.install_hooks()
-        self.addCleanup(self.reset_hooks)
 
         cmd = cmd_merge()
         # We don't care about the ouput but 'outf' should be defined
         cmd.outf = StringIOWrapper()
-        cmd.run(self.get_url() + 'branch1', directory='branch2')
+        cmd.run(self.get_url('branch1'), directory='branch2')
         self.assertEquals(1, len(self.connections))
 

=== modified file 'bzrlib/tests/commands/test_missing.py'
--- bzrlib/tests/commands/test_missing.py	2007-07-15 11:17:19 +0000
+++ bzrlib/tests/commands/test_missing.py	2007-07-20 16:57:41 +0000
@@ -17,7 +17,8 @@
 
 from bzrlib.builtins import cmd_missing
 from bzrlib.tests import StringIOWrapper
-from bzrlib.tests.TransportUtil import TestCaseWithConnectionHookedTransport
+from bzrlib.tests.transport_util import TestCaseWithConnectionHookedTransport
+
 
 class TestMissing(TestCaseWithConnectionHookedTransport):
 
@@ -29,11 +30,10 @@
         wt2.commit('empty commit too')
 
         self.install_hooks()
-        self.addCleanup(self.reset_hooks)
 
         cmd = cmd_missing()
         # We don't care about the ouput but 'outf' should be defined
         cmd.outf = StringIOWrapper()
-        cmd.run(self.get_url() + 'branch2')
+        cmd.run(self.get_url('branch2'))
         self.assertEquals(1, len(self.connections))
 

=== modified file 'bzrlib/tests/commands/test_pull.py'
--- bzrlib/tests/commands/test_pull.py	2007-07-15 11:17:19 +0000
+++ bzrlib/tests/commands/test_pull.py	2007-07-20 16:57:41 +0000
@@ -17,7 +17,8 @@
 
 from bzrlib.builtins import cmd_pull
 from bzrlib.tests import StringIOWrapper
-from bzrlib.tests.TransportUtil import TestCaseWithConnectionHookedTransport
+from bzrlib.tests.transport_util import TestCaseWithConnectionHookedTransport
+
 
 class TestPull(TestCaseWithConnectionHookedTransport):
 
@@ -27,11 +28,10 @@
         wt2 = self.make_branch_and_tree('branch2')
 
         self.install_hooks()
-        self.addCleanup(self.reset_hooks)
 
         cmd = cmd_pull()
         # We don't care about the ouput but 'outf' should be defined
         cmd.outf = StringIOWrapper()
-        cmd.run(self.get_url() + 'branch1', directory='branch2')
+        cmd.run(self.get_url('branch1'), directory='branch2')
         self.assertEquals(1, len(self.connections))
 

=== modified file 'bzrlib/tests/commands/test_push.py'
--- bzrlib/tests/commands/test_push.py	2007-07-15 11:17:19 +0000
+++ bzrlib/tests/commands/test_push.py	2007-07-20 16:57:41 +0000
@@ -16,7 +16,8 @@
 
 
 from bzrlib.builtins import cmd_push
-from bzrlib.tests.TransportUtil import TestCaseWithConnectionHookedTransport
+from bzrlib.tests.transport_util import TestCaseWithConnectionHookedTransport
+
 
 class TestPush(TestCaseWithConnectionHookedTransport):
 
@@ -24,9 +25,8 @@
         self.make_branch_and_tree('branch')
 
         self.install_hooks()
-        self.addCleanup(self.reset_hooks)
 
         cmd = cmd_push()
-        cmd.run(self.get_url() + 'remote', directory='branch')
+        cmd.run(self.get_url('remote'), directory='branch')
         self.assertEquals(1, len(self.connections))
 

=== renamed file 'bzrlib/tests/TransportUtil.py' => 'bzrlib/tests/transport_util.py'
--- bzrlib/tests/TransportUtil.py	2007-07-15 10:44:12 +0000
+++ bzrlib/tests/transport_util.py	2007-07-20 16:57:41 +0000
@@ -14,14 +14,13 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-
 from bzrlib.hooks import Hooks
-from bzrlib.tests.test_ftp_transport import TestCaseWithFTPServer
+from bzrlib.tests.test_sftp_transport import TestCaseWithSFTPServer
 from bzrlib.transport import (
     register_transport,
     unregister_transport,
     )
-from bzrlib.transport.ftp import FtpTransport
+from bzrlib.transport.sftp import SFTPTransport
 
 
 class TransportHooks(Hooks):
@@ -34,7 +33,7 @@
         self['_set_connection'] = []
 
 
-class InstrumentedTransport(FtpTransport):
+class InstrumentedTransport(SFTPTransport):
     """Instrumented transport class to test commands behavior"""
 
     hooks = TransportHooks()
@@ -51,31 +50,23 @@
             hook(self, connection, credentials)
 
 
-class TestCaseWithConnectionHookedTransport(TestCaseWithFTPServer):
+class TestCaseWithConnectionHookedTransport(TestCaseWithSFTPServer):
 
     def setUp(self):
         super(TestCaseWithConnectionHookedTransport, self).setUp()
-        self._hooks_installed = False
-
-        def cleanup():
-            # Be nice and reset hooks if the test writer forgot about them
-            if self._hooks_installed:
-                self.reset_hooks()
-
-        self.addCleanup(cleanup)
-        self.connections = []
+        self.reset_connections()
 
     def install_hooks(self):
         ConnectionHookedTransport.hooks.install_hook('_set_connection',
                                                      self.set_connection_hook)
         # Make our instrumented transport the default ftp transport
-        register_transport('ftp://', ConnectionHookedTransport)
-        self._hooks_installed = True
+        register_transport('sftp://', ConnectionHookedTransport)
+        # uninstall our hooks when we are finished
+        self.addCleanup(self.reset_hooks)
 
     def reset_hooks(self):
         InstrumentedTransport.hooks = TransportHooks()
-        unregister_transport('ftp://', ConnectionHookedTransport)
-        self._hooks_installed = False
+        unregister_transport('sftp://', ConnectionHookedTransport)
 
     def reset_connections(self):
         self.connections = []

=== modified file 'bzrlib/transport/__init__.py'
--- bzrlib/transport/__init__.py	2007-07-15 11:24:18 +0000
+++ bzrlib/transport/__init__.py	2007-07-20 16:57:41 +0000
@@ -1106,26 +1106,26 @@
     them and share the underlying, protocol specific, connection.
     """
 
-    def __init__(self, base, from_transport=None):
+    def __init__(self, base, _from_transport=None):
         """Constructor.
 
         :param base: transport root URL
 
-        :param from_transport: optional transport to build from. The built
+        :param _from_transport: optional transport to build from. The built
             transport will share the connection with this transport.
         """
-        if base[-1] != '/':
+        if not base.endswith('/'):
             base += '/'
         (self._scheme,
          self._user, self._password,
          self._host, self._port,
          self._path) = self._split_url(base)
-        if from_transport is not None:
+        if _from_transport is not None:
             # Copy the password as it does not appear in base and will be lost
-            # otherwise. It can appear in the _initial_split_url above if the
-            # user provided it on the command line. Otherwise, daughter classes
-            # will prompt the user for one when appropriate.
-            self._password = from_transport._password
+            # otherwise. It can appear in the _split_url above if the user
+            # provided it on the command line. Otherwise, daughter classes will
+            # prompt the user for one when appropriate.
+            self._password = _from_transport._password
 
         base = self._unsplit_url(self._scheme,
                                  self._user, self._password,
@@ -1133,10 +1133,10 @@
                                  self._path)
 
         super(ConnectedTransport, self).__init__(base)
-        if from_transport is None:
+        if _from_transport is None:
             self._shared_connection = _SharedConnection()
         else:
-            self._shared_connection = from_transport._shared_connection
+            self._shared_connection = _from_transport._shared_connection
 
     def clone(self, offset=None):
         """Return a new transport with root at self.base + offset
@@ -1145,9 +1145,9 @@
         that it's a cloning not a raw creation.
         """
         if offset is None:
-            return self.__class__(self.base, self)
+            return self.__class__(self.base, _from_transport=self)
         else:
-            return self.__class__(self.abspath(offset), self)
+            return self.__class__(self.abspath(offset), _from_transport=self)
 
     @staticmethod
     def _split_url(url):
@@ -1336,8 +1336,10 @@
         transport = None
         # Don't compare passwords, they may be absent from other_base or from
         # self and they don't carry more information than user anyway.
-        if (scheme, user, host, port) == (self._scheme,
-                                          self._user, self._host, self._port):
+        if (scheme == self._scheme
+            and user == self._user
+            and host == self._host
+            and port == self._port):
             if not path.endswith('/'):
                 # This normally occurs at __init__ time, but it's easier to do
                 # it now to avoid creating two transports for the same base.
@@ -1348,7 +1350,7 @@
             # We don't call clone here because the intent is different: we
             # build a new transport on a different base (which may be totally
             # unrelated) but we share the connection.
-            transport = self.__class__(other_base, self)
+            transport = self.__class__(other_base, _from_transport=self)
         return transport
 
 
@@ -1367,10 +1369,10 @@
     :param base: either a URL or a directory name.
 
     :param transports: optional reusable transports list. If not None, created
-    transports will be added to the list.
+        transports will be added to the list.
 
     :return: A new transport optionally sharing its connection with one of
-    possible_transports.
+        possible_transports.
     """
     if base is None:
         base = '.'

=== modified file 'bzrlib/transport/ftp.py'
--- bzrlib/transport/ftp.py	2007-07-15 11:24:18 +0000
+++ bzrlib/transport/ftp.py	2007-07-20 16:57:41 +0000
@@ -79,14 +79,16 @@
 # FIXME: there are inconsistencies in the way temporary errors are
 # handled. Sometimes we reconnect, sometimes we raise an exception. Care should
 # be taken to analyze the implications for write operations (read operations
-# are safe to retry). Overall even some read operations are never retried.
+# are safe to retry). Overall even some read operations are never
+# retried. --vila 20070720 (Bug #127164)
 class FtpTransport(ConnectedTransport):
     """This is the transport agent for ftp:// access."""
 
-    def __init__(self, base, from_transport=None):
+    def __init__(self, base, _from_transport=None):
         """Set the base path where files will be stored."""
         assert base.startswith('ftp://') or base.startswith('aftp://')
-        super(FtpTransport, self).__init__(base, from_transport)
+        super(FtpTransport, self).__init__(base,
+                                           _from_transport=_from_transport)
         self._unqualified_scheme = 'ftp'
         if self._scheme == 'aftp':
             self.is_active = True

=== modified file 'bzrlib/transport/http/__init__.py'
--- bzrlib/transport/http/__init__.py	2007-07-15 11:24:18 +0000
+++ bzrlib/transport/http/__init__.py	2007-07-20 16:57:41 +0000
@@ -43,14 +43,6 @@
     Transport,
     )
 
-# FIXME: There are two known cases where we open a new connection during the
-# lifetime of the transport:
-# - when an error is received from the server,
-# - when the keep-alive header reach zero.
-# This should be taken into account for the connection sharing by either
-# failing without reconnecting or inform the clones that a new connection have
-# been established.
-
 # TODO: This is not used anymore by HttpTransport_urllib
 # (extracting the auth info and prompting the user for a password
 # have been split), only the tests still use it. It should be
@@ -141,7 +133,7 @@
     # _unqualified_scheme: "http" or "https"
     # _scheme: may have "+pycurl", etc
 
-    def __init__(self, base, from_transport=None):
+    def __init__(self, base, _from_transport=None):
         """Set the base path where files will be stored."""
         proto_match = re.match(r'^(https?)(\+\w+)?://', base)
         if not proto_match:
@@ -151,7 +143,8 @@
         if impl_name:
             impl_name = impl_name[1:]
         self._impl_name = impl_name
-        super(HttpTransportBase, self).__init__(base, from_transport)
+        super(HttpTransportBase, self).__init__(base,
+                                                _from_transport=_from_transport)
         # range hint is handled dynamically throughout the life
         # of the transport object. We start by trying multi-range
         # requests and if the server returns bogus results, we
@@ -159,8 +152,8 @@
         # forget about range if the server really can't
         # understand. Once acquired, this piece of info is
         # propagated to clones.
-        if from_transport is not None:
-            self._range_hint = from_transport._range_hint
+        if _from_transport is not None:
+            self._range_hint = _from_transport._range_hint
         else:
             self._range_hint = 'multi'
 

=== modified file 'bzrlib/transport/http/_pycurl.py'
--- bzrlib/transport/http/_pycurl.py	2007-07-04 12:28:56 +0000
+++ bzrlib/transport/http/_pycurl.py	2007-07-20 16:57:41 +0000
@@ -87,8 +87,9 @@
     Python client.  Advantages include: DNS caching.
     """
 
-    def __init__(self, base, from_transport=None):
-        super(PyCurlTransport, self).__init__(base, from_transport)
+    def __init__(self, base, _from_transport=None):
+        super(PyCurlTransport, self).__init__(base,
+                                              _from_transport=_from_transport)
         if base.startswith('https'):
             # Check availability of https into pycurl supported
             # protocols

=== modified file 'bzrlib/transport/http/_urllib.py'
--- bzrlib/transport/http/_urllib.py	2007-07-04 12:28:56 +0000
+++ bzrlib/transport/http/_urllib.py	2007-07-20 16:57:41 +0000
@@ -47,10 +47,11 @@
 
     _opener_class = Opener
 
-    def __init__(self, base, from_transport=None):
-        super(HttpTransport_urllib, self).__init__(base, from_transport)
-        if from_transport is not None:
-            self._opener = from_transport._opener
+    def __init__(self, base, _from_transport=None):
+        super(HttpTransport_urllib, self).__init__(
+            base, _from_transport=_from_transport)
+        if _from_transport is not None:
+            self._opener = _from_transport._opener
         else:
             self._opener = self._opener_class()
 

=== modified file 'bzrlib/transport/remote.py'
--- bzrlib/transport/remote.py	2007-07-15 11:24:18 +0000
+++ bzrlib/transport/remote.py	2007-07-20 16:57:41 +0000
@@ -78,28 +78,29 @@
     # it. It may be even clearer to define a TestRemoteTransport that handles
     # the specific cases of providing a _client and/or a _medium, and leave
     # RemoteTransport as an abstract class.
-    def __init__(self, url, from_transport=None, medium=None, _client=None):
+    def __init__(self, url, _from_transport=None, medium=None, _client=None):
         """Constructor.
 
-        :param from_transport: Another RemoteTransport instance that this
+        :param _from_transport: Another RemoteTransport instance that this
             one is being cloned from.  Attributes such as the medium will
             be reused.
 
         :param medium: The medium to use for this RemoteTransport. This must be
-            supplied if from_transport is None.
+            supplied if _from_transport is None.
 
         :param _client: Override the _SmartClient used by this transport.  This
             should only be used for testing purposes; normally this is
             determined from the medium.
         """
-        super(RemoteTransport, self).__init__(url, from_transport)
+        super(RemoteTransport, self).__init__(url,
+                                              _from_transport=_from_transport)
 
         # The medium is the connection, except when we need to share it with
         # other objects (RemoteBzrDir, RemoteRepository etc). In these cases
         # what we want to share is really the shared connection.
 
-        if from_transport is None:
-            # If no from_transport is specified, we need to intialize the
+        if _from_transport is None:
+            # If no _from_transport is specified, we need to intialize the
             # shared medium.
             credentials = None
             if medium is None:
@@ -113,7 +114,7 @@
             self._client = _client
 
     def _build_medium(self):
-        """Create the medium if from_transport does not provide one.
+        """Create the medium if _from_transport does not provide one.
 
         The medium is analogous to the connection for ConnectedTransport: it
         allows connection sharing.
@@ -430,11 +431,8 @@
         SmartTCPClientMedium).
     """
 
-    def __init__(self, base, from_transport=None):
-        assert base.startswith('bzr://')
-        super(RemoteTCPTransport, self).__init__(base, from_transport)
-
     def _build_medium(self):
+        assert self.base.startswith('bzr://')
         if self._port is None:
             self._port = BZR_DEFAULT_PORT
         return medium.SmartTCPClientMedium(self._host, self._port), None
@@ -467,7 +465,7 @@
     HTTP path into a local path.
     """
 
-    def __init__(self, base, from_transport=None, http_transport=None):
+    def __init__(self, base, _from_transport=None, http_transport=None):
         assert base.startswith('bzr+http://')
 
         if http_transport is None:
@@ -478,7 +476,8 @@
             self._http_transport = transport.get_transport(http_url)
         else:
             self._http_transport = http_transport
-        super(RemoteHTTPTransport, self).__init__(base, from_transport)
+        super(RemoteHTTPTransport, self).__init__(
+            base, _from_transport=_from_transport)
 
     def _build_medium(self):
         # We let http_transport take care of the credentials
@@ -522,7 +521,9 @@
             http_transport = self._http_transport.clone(normalized_rel_url)
         else:
             http_transport = self._http_transport
-        return RemoteHTTPTransport(abs_url, self, http_transport=http_transport)
+        return RemoteHTTPTransport(abs_url,
+                                   _from_transport=self,
+                                   http_transport=http_transport)
 
 
 def get_test_permutations():

=== modified file 'bzrlib/transport/sftp.py'
--- bzrlib/transport/sftp.py	2007-07-15 11:24:18 +0000
+++ bzrlib/transport/sftp.py	2007-07-20 16:57:41 +0000
@@ -151,9 +151,10 @@
     # up the request itself, rather than us having to worry about it
     _max_request_size = 32768
 
-    def __init__(self, base, from_transport=None):
+    def __init__(self, base, _from_transport=None):
         assert base.startswith('sftp://')
-        super(SFTPTransport, self).__init__(base, from_transport)
+        super(SFTPTransport, self).__init__(base,
+                                            _from_transport=_from_transport)
 
     def _remote_path(self, relpath):
         """Return the path to be passed along the sftp protocol for relpath.

